from prudentia.domain import Box
from prudentia.simple import SimpleProvider, SimpleCli
from prudentia.utils import io


class LocalCli(SimpleCli):
    def __init__(self):
        SimpleCli.__init__(self)
        self.prompt = '(Prudentia > Local) '
        self.provider = LocalProvider()


class LocalProvider(SimpleProvider):
    NAME = 'local'

    def __init__(self):
        super(LocalProvider, self).__init__(self.NAME)

    @staticmethod
    def _prepare(box):
        box.transport = 'local'
        box.use_prudentia_lib = True
        return box

    def define_box(self):
        playbook = io.input_path('playbook path')
        hostname = self.fetch_box_hosts(playbook)
        name = io.input_value('box name', self.suggest_name(hostname))
        return Box(name, playbook, hostname, '127.0.0.1')

    def redefine_box(self, previous_box):
        playbook = io.input_path('playbook path', previous_box.playbook)
        hostname = self.fetch_box_hosts(playbook)
        return Box(previous_box.name, playbook, hostname, previous_box.ip)

    def provision(self, box, tags):
        super(LocalProvider, self).provision(LocalProvider._prepare(box), tags)

    def facts(self, box, regex='*'):
        return super(LocalProvider, self).facts(LocalProvider._prepare(box), regex)



#!/usr/bin/python

import os
from ansible.errors import AnsibleError
from ansible.plugins.action import ActionBase

__metaclass__ = type

ANSIBLE_HASHI_VAULT_ADDR = 'http://127.0.0.1:8200'

if os.getenv('VAULT_ADDR') is not None:
    ANSIBLE_HASHI_VAULT_ADDR = os.environ['VAULT_ADDR']

if os.getenv('VAULT_TOKEN') is not None:
    ANSIBLE_HASHI_VAULT_TOKEN = os.environ['VAULT_TOKEN']


class HashiVault:
    def __init__(self, logger=None, **kwargs):
        try:
            import hvac
        except ImportError:
            AnsibleError("Please pip install hvac to use this module")

        self.url = kwargs.get('url', ANSIBLE_HASHI_VAULT_ADDR)

        self.token = kwargs.get('token', ANSIBLE_HASHI_VAULT_TOKEN)
        if self.token is None:
            raise AnsibleError("No Vault Token specified")

        self.value = kwargs.get('value', None)

        # split secret arg, which has format 'secret/hello:value'
        # into secret='secret/hello' and secret_field='value'
        s = kwargs.get('secret')
        if s is None:
            raise AnsibleError("No secret specified")

        s_f = s.split(':')
        self.secret = s_f[0]
        if len(s_f) >= 2:
            self.secret_field = s_f[1]
        else:
            self.secret_field = 'value'

        self.client = hvac.Client(url=self.url, token=self.token)

        if self.client.is_authenticated():
            pass
        else:
            raise AnsibleError("Invalid Hashicorp Vault Token Specified")

    def get(self):
        data = self.client.read(self.secret)
        if data is None:
            raise AnsibleError("The secret %s doesn't seem to exist"
                               % self.secret)

        if self.secret_field == '':  # secret was specified with trailing ':'
            return data['data']

        if self.secret_field not in data['data']:
            raise AnsibleError("The secret %s does not contain the field '%s'."
                               % (self.secret, self.secret_field))

        return data['data'][self.secret_field]

    def write(self):
        if self.value is None:
            raise AnsibleError("Value is required for write operation")
        return self.client.write(self.secret, value=self.value)

    def delete(self):
        return self.client.delete(self.secret)


class ActionModule(ActionBase):
    ''' Write operations with hashicorp vault '''

    VALID_ARGS = set(['action', 'url', 'token', 'secret', 'value'])
    VALID_ACTIONS = set(['read', 'write', 'delete'])

    def run(self, tmp=None, task_vars=None):
        #  self._display.warning('This is how you can debug')

        module_args = dict()
        for arg in self._task.args:
            if arg not in self.VALID_ARGS:
                return {"failed": True, "msg": "'%s' is not a valid option in hashi_vault" % arg}
            else:
                module_args[arg] = self._task.args.get(arg)

        if task_vars is None:
            task_vars = module_args.deepcopy(module_args)

        result = super(ActionModule, self).run(tmp, task_vars)
        result['_ansible_verbose_always'] = True

        action = self._task.args.get('action', 'read')

        vault_conn = HashiVault(self._display, **module_args)

        if action == 'read':
            result['value'] = vault_conn.get()
            result['changed'] = True
        elif action == 'write':
            result['status'] = vault_conn.write()
            result['changed'] = True
        elif action == 'delete':
            result['status'] = vault_conn.delete()
            result['changed'] = True
        else:
            return {"failed": True, "msg": "'%s' is not a valid action in hashi_vault" % action}

        return result



import os
import tempfile
from datetime import datetime

import ansible.constants as C
from ansible.errors import AnsibleError
from ansible.executor.playbook_executor import PlaybookExecutor
from ansible.executor.task_queue_manager import TaskQueueManager
from ansible.parsing.vault import get_file_vault_secret
from ansible.playbook import Play
from bunch import Bunch
from prudentia.utils import io

try:
    from __main__ import display
except ImportError:
    from ansible.utils.display import Display
    display = Display()

VERBOSITY = 0


def split_vault_id(vault_id):
    # return (before_@, after_@)
    # if no @, return whole string as after_
    if '@' not in vault_id:
      return (None, vault_id)

    parts = vault_id.split('@', 1)
    ret = tuple(parts)
    return ret


def build_vault_ids(vault_ids, vault_password_files=None):
    vault_password_files = vault_password_files or []
    vault_ids = vault_ids or []

    # convert vault_password_files into vault_ids slugs
    for password_file in vault_password_files:
        id_slug = u'%s@%s' % (C.DEFAULT_VAULT_IDENTITY, password_file)
        vault_ids.append(id_slug)
    return vault_ids


def load_vault_secrets(loader, vault_ids, vault_password_files=None):
    vault_secrets = []

    vault_password_files = vault_password_files or []
    if C.DEFAULT_VAULT_PASSWORD_FILE:
        vault_password_files.append(C.DEFAULT_VAULT_PASSWORD_FILE)

    vault_ids = build_vault_ids(vault_ids,
                                vault_password_files)

    for vault_id_slug in vault_ids:
        vault_id_name, vault_id_value = split_vault_id(vault_id_slug)
        # assuming anything else is a password file
        display.vvvvv('Reading vault password file: %s' % vault_id_value)
        # read vault_pass from a file
        file_vault_secret = get_file_vault_secret(filename=vault_id_value,
                                                  vault_id=vault_id_name,
                                                  loader=loader)
        # an invalid password file will error globally
        try:
            file_vault_secret.load()
        except AnsibleError as exc:
            display.warning('Error in vault password file loading (%s): %s' % (vault_id_name, exc))
            raise

        if vault_id_name:
            vault_secrets.append((vault_id_name, file_vault_secret))
        else:
            vault_secrets.append((C.DEFAULT_VAULT_IDENTITY, file_vault_secret))
    return vault_secrets


def run_playbook(playbook_file, inventory_file, loader, remote_user=C.DEFAULT_REMOTE_USER,
                 remote_pass=C.DEFAULT_REMOTE_PASS, transport=C.DEFAULT_TRANSPORT,
                 extra_vars=None, only_tags=None):
    variable_manager = get_variable_manager(loader)
    variable_manager.extra_vars = {} if extra_vars is None else extra_vars

    loader._FILE_CACHE = dict()  # clean up previously read and cached files
    inv = get_inventory(loader=loader, variable_manager=variable_manager, inventory_file=inventory_file)
    variable_manager.set_inventory(inv)

    options = default_options(remote_user, transport, only_tags)

    vault_ids = options.vault_identity_list
    default_vault_ids = C.DEFAULT_VAULT_IDENTITY_LIST
    vault_ids = default_vault_ids + vault_ids

    loader.set_vault_secrets(
        load_vault_secrets(
            loader=loader,
            vault_ids=vault_ids,
            vault_password_files=options.vault_password_files
        )
    )

    display.verbosity = options.verbosity
    pbex = PlaybookExecutor(
        playbooks=[playbook_file],
        inventory=inv,
        variable_manager=variable_manager,
        loader=loader,
        options=options,
        passwords={'conn_pass': remote_pass} if remote_pass else {}
    )

    start = datetime.now()
    results = pbex.run()
    print ("Play run took {0} minutes\n".format((datetime.now() - start).seconds / 60))

    return results == 0


def run_play(play_ds, inventory_file, loader, remote_user, remote_pass, transport, extra_vars=None):
    variable_manager = get_variable_manager(loader)
    variable_manager.extra_vars = {} if extra_vars is None else extra_vars

    loader._FILE_CACHE = dict()  # clean up previously read and cached files
    inv = get_inventory(loader=loader, variable_manager=variable_manager, inventory_file=inventory_file)
    variable_manager.set_inventory(inv)

    play = Play.load(play_ds, variable_manager=variable_manager, loader=loader)

    tqm = None
    result = 1
    try:
        options = default_options(remote_user, transport)
        display.verbosity = options.verbosity
        tqm = TaskQueueManager(
            inventory=inv,
            variable_manager=variable_manager,
            loader=loader,
            options=options,
            passwords={'conn_pass': remote_pass} if remote_pass else {},
            stdout_callback='minimal',
            run_additional_callbacks=C.DEFAULT_LOAD_CALLBACK_PLUGINS,
            run_tree=False,
        )
        result = tqm.run(play)
    except Exception as ex:
        io.track_error('cannot run play', ex)
    finally:
        if tqm:
            tqm.cleanup()

    return result == 0


def get_variable_manager(loader):
    try:
        # Ansible 2.4
        from ansible.vars.manager import VariableManager
        return VariableManager(loader)
    except:
        # Ansible 2.3
        from ansible.vars import VariableManager
        return VariableManager()


def get_inventory(loader, variable_manager, inventory_file):
    try:
        # Ansible 2.4
        from ansible.inventory.manager import InventoryManager
        return InventoryManager(
            loader=loader,
            sources=inventory_file
        )
    except:
        # Ansible 2.3
        from ansible import inventory
        inventory.HOSTS_PATTERNS_CACHE = {}  # clean up previous cached hosts
        return inventory.Inventory(
            loader=loader,
            variable_manager=variable_manager,
            host_list=inventory_file
        )


def default_options(remote_user, transport, only_tags=None):
    if only_tags is None:
        only_tags = []

    options = Bunch()
    options.remote_user = remote_user
    options.connection = transport
    options.tags = only_tags
    options.listhosts = False
    options.listtasks = False
    options.listtags = False

    options.syntax = None
    options.private_key_file = None
    options.ssh_common_args = None
    options.sftp_extra_args = None
    options.scp_extra_args = None
    options.ssh_extra_args = ''
    options.ssh_common_args = ''
    options.ssh_args = ''
    options.become = None
    options.become_method = 'sudo'
    options.become_user = 'root'

    options.verbosity = VERBOSITY
    options.check = None
    options.diff = None

    options.module_path = None
    options.forks = 5

    options.vault_identity_list = []
    options.vault_id_match = C.DEFAULT_VAULT_ID_MATCH
    options.vault_identity = C.DEFAULT_VAULT_IDENTITY
    options.vault_password_files = C.DEFAULT_VAULT_PASSWORD_FILE

    return options


def generate_inventory(box):
    if box.ip.startswith("./") or box.ip.startswith("/"):
        tmp_inventory = box.ip
    else:
        fd, tmp_inventory = tempfile.mkstemp(prefix='prudentia-inventory-', text=False)
        try:
            os.write(fd, box.inventory().encode("utf-8"))
        except IOError as ex:
            io.track_error('cannot write inventory file', ex)
        finally:
            os.close(fd)
    return tmp_inventory


def create_user(box, loader):
    res = False

    user = box.remote_user
    if 'root' not in user:
        if 'jenkins' in user:
            user_home = '/var/lib/jenkins'
        else:
            user_home = '/home/' + user

        sudo_user_play = os.path.join(io.prudentia_python_dir(), 'tasks', 'add-sudo-user.yml')
        res = run_play(
            play_ds=dict(
                hosts=box.hostname,
                gather_facts='no',
                tasks=loader.load_from_file(sudo_user_play)
            ),
            inventory_file=generate_inventory(box),
            loader=loader,
            remote_user='root',
            remote_pass=box.get_remote_pwd(),
            transport=box.get_transport(),
            extra_vars={'ssh_seed_user': 'root', 'user': user, 'group': user, 'home': user_home}
        )
    else:
        print ('Root user cannot be created!')

    return res


def gather_facts(box, filter_value, loader):
    return run_play(
        play_ds=dict(
            hosts=box.hostname,
            gather_facts='no',
            tasks=[{'setup': 'filter={0}'.format(filter_value)}]
        ),
        inventory_file=generate_inventory(box),
        loader=loader,
        remote_user=box.get_remote_user(),
        remote_pass=box.get_remote_pwd(),
        transport=box.get_transport()
    )



from abc import abstractmethod, ABCMeta

from prudentia.simple import SimpleCli, SimpleProvider


class FactoryCli(SimpleCli):
    @staticmethod
    def help_create():
        print ("Creates the box.\n")

    def complete_create(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_create(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.create(box)

    @staticmethod
    def help_start():
        print ("Starts the box.\n")

    def complete_start(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_start(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.start(box)

    @staticmethod
    def help_restart():
        print ("Restarts the box.\n")

    def complete_restart(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_restart(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.stop(box)
            self.provider.start(box)

    @staticmethod
    def help_phoenix():
        print ("Regenerates a box: destroy -> create -> provision.\n")

    def complete_phoenix(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_phoenix(self, line):
        tokens = line.split(' ')
        box = self.provider.get_box(tokens[0])
        if box:
            self.provider.rebuild(box)
            self.provider.provision(box, tokens[1:])

    @staticmethod
    def help_stop():
        print ("Stops the box.\n")

    def complete_stop(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_stop(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.stop(box)

    @staticmethod
    def help_destroy():
        print ("Destroys the box.\n")

    def complete_destroy(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_destroy(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.destroy(box)

    @staticmethod
    def help_status():
        print ("Check the status of the box.\n")

    def complete_status(self, text, line, begidx, endidx):
        return self.complete_box_names(text, line, begidx, endidx)

    def do_status(self, line):
        box = self.provider.get_box(line)
        if box:
            self.provider.status(box)


class FactoryProvider(SimpleProvider):
    __metaclass__ = ABCMeta

    def add_box(self, box):
        self.create(box)
        super(FactoryProvider, self).add_box(box)

    @abstractmethod
    def create(self, box):
        pass

    @abstractmethod
    def start(self, box):
        pass

    @abstractmethod
    def stop(self, box):
        pass

    def remove_box(self, box):
        box = super(FactoryProvider, self).remove_box(box)
        self.destroy(box)
        return box

    @abstractmethod
    def destroy(self, box):
        pass

    def rebuild(self, box):
        self.stop(box)
        self.destroy(box)
        self.create(box)
        self.start(box)

    @abstractmethod
    def status(self, box):
        pass



from __future__ import (absolute_import, division, print_function)
import os
from ansible.errors import AnsibleError
from ansible.plugins.lookup import LookupBase

__metaclass__ = type

ANSIBLE_HASHI_VAULT_ADDR = 'http://127.0.0.1:8200'
ANSIBLE_HASHI_VAULT_TOKEN = None

if os.getenv('VAULT_ADDR') is not None:
    ANSIBLE_HASHI_VAULT_ADDR = os.environ['VAULT_ADDR']

if os.getenv('VAULT_TOKEN') is not None:
    ANSIBLE_HASHI_VAULT_TOKEN = os.environ['VAULT_TOKEN']


class HashiVault:
    def __init__(self, logger, **kwargs):
        try:
            import hvac
        except ImportError:
            AnsibleError("Please pip install hvac to use this module")

        self.url = kwargs.get('url', ANSIBLE_HASHI_VAULT_ADDR)

        self.token = kwargs.get('token', ANSIBLE_HASHI_VAULT_TOKEN)
        if self.token is None:
            raise AnsibleError("No Vault Token specified")

        # split secret arg, which has format 'secret/hello:value'
        # into secret='secret/hello' and secret_field='value'
        s = kwargs.get('secret')
        if s is None:
            raise AnsibleError("No secret specified")

        s_f = s.split(':')
        self.secret = s_f[0]
        if len(s_f) >= 2:
            self.secret_field = s_f[1]
        else:
            self.secret_field = 'value'

        logger.warning('%s %s' % (self.url, self.token))
        self.client = hvac.Client(url=self.url, token=self.token)

        if self.client.is_authenticated():
            pass
        else:
            raise AnsibleError("Invalid Hashicorp Vault Token Specified")

    def get(self):
        data = self.client.read(self.secret)
        if data is None:
            raise AnsibleError("The secret %s doesn't seem to exist"
                               % self.secret)

        if self.secret_field == '':  # secret was specified with trailing ':'
            return data['data']

        if self.secret_field not in data['data']:
            raise AnsibleError("The secret %s does not contain the field '%s'."
                               % (self.secret, self.secret_field))

        return data['data'][self.secret_field]


class LookupModule(LookupBase):
    def run(self, terms, variables, **kwargs):
        vault_args = terms[0].split(' ')
        vault_dict = {}
        ret = []

        for param in vault_args:
            try:
                key, value = param.split('=')
            except ValueError as e:
                raise AnsibleError("hashi_vault plugin needs key=value pairs, but received %s %s"
                                   % (terms, e.message))
            vault_dict[key] = value

        vault_conn = HashiVault(self._display, **vault_dict)

        for term in terms:
            key = term.split()[0]
            value = vault_conn.get()
            ret.append(value)

        if 'write_to_file' in vault_dict.keys():
            text_file = open(vault_dict['write_to_file'], "w")
            text_file.write(value)
            text_file.close()

        return ret
